// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Standard
#include <iostream>

// STL
#include <string>

#include <anna/core/functions.hpp>
#include <anna/core/core.hpp>
#include <anna/core/RuntimeException.hpp>

using namespace anna;


void _exit(const std::string & msg) {
   std::cout << std::endl << msg << std::endl;
   exit(-1);
}


#define TRY_PRINT(d,a)   { try { std::cout << (d) << (a) << std::endl; } catch (anna::RuntimeException &ex) { ex.trace(); };}



int main(int argc, char** argv) {
   Logger::setLevel(Logger::Information);
//   Logger::setLevel(Logger::Debug);
   Logger::initialize("IP Managing", new TraceWriter("file.trace", 2048000));
   std::string ip1, ip2, preffixLength;
   std::cout << "Let's read two IP addresses:\n\n";
   std::cout << "Input ip: " ;
   std::cin >> ip1;
   std::cout << "Input ip: " ;
   std::cin >> ip2;
   std::cout << "Input ipv6 preffix length for last input: " ;
   std::cin >> preffixLength;
   std::cout << std::endl;
   std::cout << anna::functions::highlightJustify("Input 1");
   std::cout << "Address family: ";

   if (anna::functions::isIPv4(ip1)) std::cout << "Pure IPv4" << std::endl;

   if (anna::functions::isIPv4(ip1, anna::functions::IPv4Type::Compatible)) std::cout << "IPv4 - compatible" << std::endl;

   if (anna::functions::isIPv4(ip1, anna::functions::IPv4Type::Mapped)) std::cout << "IPv4 - mapped" << std::endl;

   if (anna::functions::isIPv6(ip1)) std::cout << "IPv6" << std::endl;

   TRY_PRINT("IPv4 to IPv6 conversion: ", anna::functions::IPv4To6(ip1));
   TRY_PRINT("Normalization: ", anna::functions::normalizeIP(ip1));
   anna::DataBlock db1 = anna::functions::ipAsRaw(ip1);
   db1 = anna::functions::rawIpPresentationAsRaw("AABBCCDD");
   TRY_PRINT("Datablock encoding: ", anna::functions::asString(db1));
   std::string hrRawPresentation1 = anna::functions::rawIpAsRawIpPresentation(db1);
   TRY_PRINT("Datablock as Raw Presentation (human-readable): ", hrRawPresentation1);
   exit(-1);
   TRY_PRINT("Datablock encoding from former: ", anna::functions::asString(db1));
   TRY_PRINT("Former decoded and normalized: ", anna::functions::rawIpAsString(db1, true));
   TRY_PRINT("Former decoded not normalized: ", anna::functions::rawIpAsString(db1));
   TRY_PRINT("Abbreviated: ", anna::functions::abbreviateIP(ip1));
   std::cout << std::endl;
   std::cout << anna::functions::highlightJustify("Input 2");
   std::cout << "Address family: ";

   if (anna::functions::isIPv4(ip2)) std::cout << "Pure IPv4" << std::endl;

   if (anna::functions::isIPv4(ip2, anna::functions::IPv4Type::Compatible)) std::cout << "IPv4 - compatible" << std::endl;

   if (anna::functions::isIPv4(ip2, anna::functions::IPv4Type::Mapped)) std::cout << "IPv4 - mapped" << std::endl;

   if (anna::functions::isIPv6(ip2)) std::cout << "IPv6" << std::endl;

   TRY_PRINT("IPv4 to IPv6 conversion: ", anna::functions::IPv4To6(ip2));
   TRY_PRINT("Normalization: ", anna::functions::normalizeIP(ip2));
   anna::DataBlock db2 = anna::functions::ipAsRaw(ip2);
   TRY_PRINT("Datablock encoding: ", anna::functions::asString(db2));
   std::string hrRawPresentation2 = anna::functions::rawIpAsRawIpPresentation(db2);
   db2 = anna::functions::rawIpPresentationAsRaw(hrRawPresentation2);
   TRY_PRINT("Datablock as Raw Presentation (human-readable): ", hrRawPresentation2);
   TRY_PRINT("Datablock encoding from former: ", anna::functions::asString(db2));
   TRY_PRINT("Former decoded and normalized: ", anna::functions::rawIpAsString(db2, true));
   TRY_PRINT("Former decoded not normalized: ", anna::functions::rawIpAsString(db2));
   TRY_PRINT("Abbreviated: ", anna::functions::abbreviateIP(ip2));
   std::cout << std::endl;
   std::cout << anna::functions::highlightJustify("Input 1 and 2 ARE EQUIVALENT ?");
   TRY_PRINT("", anna::functions::sameIP(ip1, ip2) ? "YES" : "NO");
   std::cout << anna::functions::highlightJustify("Input 1 and preffixed-2 ARE EQUIVALENT ?");
   std::string preffixedIp2 = ip2;
   preffixedIp2 += "/";
   preffixedIp2 += preffixLength;
   TRY_PRINT("", anna::functions::matchIPv6(ip1, preffixedIp2) ? "YES" : "NO");
   std::cout << std::endl;
   std::cout << std::endl;
   std::cout << std::endl;
}

