import pytest

def test_001_given_diameter_hex_representation_i_want_to_send_message_to_diameter_entity_and_check_it_fails_for_auto_node(resources, admlc):

  responseBodyRef = { "success":"true", "response":"Working node has been set to automatic" }

  # Send POST
  response = admlc.post("/node-auto")

  # Verify response
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

  diameterHex = resources("aar.hex")
  #requestBody = resources("aaa-diameterHex.json.in", diameterHex=diameterHex.rstrip())
  requestBodyDict = { "diameterHex":"{}".format(diameterHex.rstrip()) }

  responseBodyRef = { "success":"false", "response":"Internal error (check ADML traces): invalid operated host" }

  # Send POST
  response = admlc.postDict("/sendhex2e", requestBodyDict)

  # Verify response
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

  # IT FAILS BECAUSE MESSAGE SENT HAS OCS3 AS ORIGIN HOST, AND THIS IS NOT REGISTERED, SO node-auto COULD NOT DEDUCE THE CORRECT RESOURCE

@pytest.mark.xfail(reason="Server must receive client connection to be available. Component test is very fast and this could fail the first time. Wait and test again. TODO: wait for available status after 'services' loading.")
def test_002_given_diameter_hex_representation_i_want_to_send_message_to_diameter_entity_and_check_it_succeed_for_correct_node(resources, admlc):

  requestBody = { "name":"afHost.afRealm.com" }
  responseBodyRef = { "success":"true", "response":"Forced node is now 'afHost.afRealm.com'" }

  # Send POST
  response = admlc.postDict("/node", requestBody)

  # Verify response
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

  diameterHex = resources("aar.hex")
  #requestBody = resources("aaa-diameterHex.json.in", diameterHex=diameterHex.rstrip())
  requestBodyDict = { "diameterHex":"{}".format(diameterHex.rstrip()) }

  responseBodyRef = { "success":"true", "response":"Operation processed" }

  # Send POST
  response = admlc.postDict("/sendhex2e", requestBodyDict)

  # Verify response
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

@pytest.mark.xfail(reason="Server must receive client connection to be available. Component test is very fast and this could fail the first time. Wait and test again. TODO: wait for available status after 'services' loading.")
def test_003_given_diameter_hex_representation_i_want_to_send_message_to_diameter_client_and_check_it_succeed_for_correct_node(resources, admlc):

#  # WE HAVE TO CONFIGURE THE CLIENT AS OPERATED HOST, BECAUSE HEX SENDING PROCEDURE IS NOT GOING TO UPDATE OPERATED HOST WITH CODEC:
#  requestBody = { "name":"afHost.afRealm.com" }
#  responseBodyRef = { "success":"true", "response":"Forced node is now 'afHost.afRealm.com'" }
  requestBody = { "name":"ownHostId.operatorRealm.com" }
  responseBodyRef = { "success":"true", "response":"Forced node is now 'ownHostId.operatorRealm.com'" }

  # Send POST
  response = admlc.postDict("/node", requestBody)

  # Verify response
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

  # NORMALLY, AAR IS NOT THE KIND OF MESSAGE TO SEND AS DIAMETER SERVER BUT RAR, etc, BUT THIS IS VALID FOR OUR TESTING

  # We will use a message which origin-host and destination-host fits the real configured ones in services, and also is ready
  #  to be sent from local diameter server to the client:
  diameterHex = resources("aarFromOwnToAF.hex")
  #requestBody = resources("aaa-diameterHex.json.in", diameterHex=diameterHex.rstrip())
  requestBodyDict = { "diameterHex":"{}".format(diameterHex.rstrip()) }

  responseBodyRef = { "success":"true", "response":"Operation processed" }

  # Send POST
  response = admlc.postDict("/sendhex2c", requestBodyDict)

  # Verify response
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

