// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_RoundRobinDelivery_hpp
#define anna_comm_RoundRobinDelivery_hpp

#include <anna/comm/Service.hpp>

namespace anna {

namespace comm {
class Resource;
}

namespace comm {

/**
   Servicio de reparto de carga mediante un algoritmo RoundRobin, es decir, cada uno de los mensajes enviados tiene como
   destino uno de los recursos definidos, cuando en el mensaje i-esimo se alcance el final de la lista de recursos
   disponibles se vuelve a empezar por el primero.
*/
class RoundRobinDelivery : public comm::Service {
public:
  /**
     Constructor.
     \param name Nombre logico del servicio de reparto de carga.
     \param isCritical Indica las caracteristicas del reparto de carga. Debe valor \em true si el
     reparto de carga a crear es critico o \em false en otro caso.
  */
  RoundRobinDelivery(const char* name, const bool isCritical) :
    comm::Service(name, isCritical),
    a_iiserver(0)
  {;}

  std::string asString() const throw();

  xml::Node* asXML(xml::Node* parent) const throw();

  /**
     Metodo que devuelve el nombre completo de este selector de recursos.
     Para evitar ambiguedades este nombre incluye la lista completa de \em namespaces
     a los que pertenece la clase.
     \return Una cadena con el nombre de este selector.
  */
  static const char* className() throw() { return "anna::comm::RoundRobinDelivery"; }

protected:
  void do_initialize() throw(RuntimeException);

private:
  //------------------------------------------------------------------------------------
  // - a_iiserver: iterador que apunta al servidor que debemos usar.
  //------------------------------------------------------------------------------------
  iterator a_iiserver;

  comm::Resource* do_apply() throw(RuntimeException);
  iterator advance() throw();
};

}
}

#endif

