// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_ExclusiveHash_hpp
#define anna_core_util_ExclusiveHash_hpp

#include <map>

namespace anna {

/**
   Template para calcular de forma univoca valores numericos del tipo de dato T.
   Un valor T puede tener valores distintos para instancias distintas de ExclusiveHash.
   La instancia T debe ser capaz de actuar como idice de una map<T,X>.

   \param T Tipo de dato que queremos codificar numericamente.
*/
template < class T, class _K = unsigned long > class ExclusiveHash {
  typedef std::map <T, _K> container;
  typedef typename container::value_type value_type;

public:
  /**
     Constructor.
  */
  ExclusiveHash() : a_value(0) {;}

  /**
     Calcula la funcion hash exclusiva para la instancia recibida como parametro.
     \param t Instancia a la que vamos a calcular un valor numerico.
  */
  _K calcule(const T& t)
  throw() {
    typename container::iterator ii;
    _K result = 0;

    if((ii = a_container.find(t)) == a_container.end()) {
      result = a_value ++;
      a_container.insert(value_type(t, result));
    } else
      result = ii->second;

    return result;
  }

private:
  container a_container;
  _K a_value;
};

}

#endif
