// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_Millisecond_hpp
#define anna_core_util_Millisecond_hpp

#include <time.h>

#include <string>

#include <anna/core/util/defines.hpp>
#include <anna/core/RuntimeException.hpp>

namespace anna {

class Second;
class Microsecond;

class Millisecond {
public:
  typedef S64 type_t;

  /**
   * Constructor
   */
  Millisecond() : a_value(0) {;}

  /**
   * Constructor.
   * \param value Valor inicial de esta instancia.
   */
  explicit Millisecond(const type_t value) : a_value(value) {;}

  /**
   * Constructor copia.
   * \param other Instancia de la que copiar.
   */
  Millisecond(const Millisecond& other) : a_value(other.a_value) {;}

  /**
   * Constructor copia.
   * \param other Instancia de la que copiar.
   */
  Millisecond(const Second& other);

  /**
   * Constructor copia.
   * \param other Instancia de la que copiar.
   */
  Millisecond(const Microsecond& other);

  /**
   * Conversor a numrico.
   * \return El valor asociado a esta instancia.
   */
  operator type_t () const throw() { return a_value; }

  /**
   * \internal
   */
  type_t& refValue() throw() { return a_value; }

  Millisecond& operator= (const type_t other) throw() { a_value = other; return *this; }

  Millisecond& operator= (const Millisecond& other) throw() { a_value = other.a_value; return *this; }

  Millisecond& operator= (const Second& other) throw();

  Millisecond& operator= (const Microsecond& other) throw();

  bool operator== (const Millisecond& other) const throw() { return a_value == other.a_value; }

  bool operator== (const Second& other) const throw();

  bool operator== (const Microsecond& other) const throw();

  bool operator!= (const Millisecond& other) const throw() { return a_value != other.a_value; }

  bool operator!= (const Second& other) const throw();

  bool operator!= (const Microsecond& other) const throw();

  bool operator> (const Millisecond& other) const throw() { return a_value > other.a_value; }

  bool operator> (const Second& other) const throw();

  bool operator> (const Microsecond& other) const throw();

  bool operator< (const Millisecond& other) const throw() { return a_value < other.a_value; }

  bool operator< (const Second& other) const throw();

  bool operator< (const Microsecond& other) const throw();

  bool operator>= (const Millisecond& other) const throw() { return a_value >= other.a_value; }

  bool operator>= (const Second& other) const throw() { return (operator==(other) == true) ? true : operator>(other); }

  bool operator>= (const Microsecond& other) const throw() { return (operator==(other) == true) ? true : operator>(other); }

  bool operator<= (const Millisecond& other) const throw() { return a_value <= other.a_value; }

  bool operator<= (const Second& other) const throw() { return (operator==(other) == true) ? true : operator<(other); }

  bool operator<= (const Microsecond& other) const throw() { return (operator==(other) == true) ? true : operator<(other); }

  Millisecond& operator+= (const Millisecond& other) throw() { a_value += other.a_value; return *this; }

  Millisecond& operator-= (const Millisecond& other) throw() {(a_value > other.a_value) ? (a_value -= other.a_value) : (a_value = 0); return *this; }

  /**
   * Devuelve el valor asociado a esta instancia.
   * \return el valor asociado a esta instancia.
   */
  type_t getValue() const throw() { return a_value; }

  /**
   * Si el valor de esta instancia es positivo devuelve el valor asociado a esta instancia en una estructura de
   * time \em timeval usada habitualmente para temporizar operaciones a nivel de SO, en otro caso retorna NULL.
   * \param tv Instancia sobre la que guardar el valor en caso de que se ste tenga un valor positivo.
   * \return Si el valor de esta instancia es positivo devuelve el valor asociado a esta instancia en una estructura de
   * time \em timeval usada habitualmente para temporizar operaciones a nivel de SO, en otro caso retorna NULL.
   */
  timeval* getTimeVal(timeval& tv) const throw();

  /**
   * Devuelve la hora actual de sistema expresada en milisegundos transcurridos desde el 1 de Enero de 1970
   * \return la hora actual de sistema expresada en milisegundos transcurridos desde el 1 de Enero de 1970
   */
  static Millisecond getTime() throw();

  /**
   * Devuelve una cadena con el valor de esta instancia y las unidades "ms".
   * \return una cadena con el valor de esta instancia y las unidades "ms".
   */
  std::string asString() const throw();

  /**
   * Obtiene los microsegundos del valor contenido en la cadena recibida como parmetro.
   * \param value Cadena que contiene los microsegundos habr sido obtenida con #asString.
   * \return los microsegundos del valor contenido en la cadena recibida como parmetro.
   */
  static Millisecond fromString(const std::string& value) throw(RuntimeException);

private:
  type_t a_value;

  friend class Second;
  friend class Microsecond;

  friend class Millisecond operator / (const Millisecond& left, const Millisecond& right) throw();
  friend class Millisecond operator + (const Millisecond& left, const Millisecond& right) throw();
  friend class Millisecond operator - (const Millisecond& left, const Millisecond& right) throw();
  friend class Millisecond operator / (const Millisecond& left, const int right) throw();
  friend class Millisecond operator / (const Millisecond& left, const unsigned int right) throw();
  friend class Millisecond operator *(const Millisecond& left, const int right) throw();
};

inline Millisecond operator / (const Millisecond& left, const Millisecond& right)
throw() {
  return Millisecond(left.a_value / right.a_value);
}

inline Millisecond operator + (const Millisecond& left, const Millisecond& right)
throw() {
  return Millisecond(left.a_value + right.a_value);
}

inline Millisecond operator - (const Millisecond& left, const Millisecond& right)
throw() {
  return Millisecond(left.a_value - right.a_value);
}

inline Millisecond operator / (const Millisecond& left, const int right)
throw() {
  return Millisecond(left.a_value / right);
}

inline Millisecond operator / (const Millisecond& left, const unsigned int right)
throw() {
  return Millisecond(left.a_value / right);
}

inline Millisecond operator *(const Millisecond& left, const int right)
throw() {
  return Millisecond(left.a_value * right);
}

}

#endif
