// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_codec_Format_hpp
#define anna_diameter_codec_Format_hpp


#include <anna/core/RuntimeException.hpp>
#include <anna/core/define.autoenum.hpp>

// STL
#include <string>


namespace anna {

namespace diameter {

namespace codec {

/**
* Diameter RFC3588 Avp Data Format definition
*/
struct Format {
  enum _v {
    None = -1,        // Initialized

// Reserved
    Unknown,          // Unregistered at diameter dictionary
    Any,              // used on generic AVP
// RFC 3588
    OctetString,      // S8*  n (possible zero-padding)
    Integer32,        // S32  4
    Integer64,        // S64  8
    Unsigned32,       // U32  4
    Unsigned64,       // U64  8
    Float32,          // F32  4
    Float64,          // F64  8
    Grouped,          // U32* 4*n
    Address,          // S8*  4 (IPv4) or 16 (IPv6)
    Time,             // S8*  4
    UTF8String,       // S8*  n (possible zero-padding)
    DiameterIdentity, // S8*  n (possible zero-padding)
    DiameterURI,      // S8*  n (possible zero-padding)
    Enumerated,       // S32  4
    IPFilterRule,     // S8*  n (possible zero-padding)
    QoSFilterRule     // S8*  n (possible zero-padding)
  };

  anna_declare_enum(diameter::codec::Format);

  /**
  * Format description
  *
  * @param v Format type
  *
  * @return Format description
  */
  static const char* asText(const Format::_v v) throw(anna::RuntimeException) {
    return asCString(v);
  }


  /**
  * Boolean about if format type is reserved (API internal use)
  *
  * @param formatName Format text name
  *
  * @return Boolean 'true' if reserved for internal use, 'false' if is user format type (RFC 3588 or application-specific)
  */
  static bool isReserved(const std::string & formatName) {
    if(formatName == asText(Format::Unknown)) return true;

    if(formatName == asText(Format::Any)) return true;

    return false;
  }


  /**
  * Boolean about if format type belongs to RFC 3588
  * @param formatName Format text name
  * @return Boolean 'true' if belongs to RFC 3588, 'false' if is application-specific)
  */
  static bool isRFC3588(const std::string & formatName) {
    if(isReserved(formatName)) return false;

    return (asEnum(formatName) != None);
  }

};


}
}
}


#endif

