// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_stack_Vendor_hpp
#define anna_diameter_stack_Vendor_hpp

// Local
#include <anna/config/defines.hpp>
#include <anna/diameter/defines.hpp>

#include <anna/config/defines.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/core/define.autoenum.hpp>

// STL
#include <string>



namespace anna {
namespace xml {
class Node;
}
}



namespace anna {

namespace diameter {

namespace stack {


//------------------------------------------------------------------------------
//----------------------------------------------------------------- class Vendor
//------------------------------------------------------------------------------
/**
* Vendor data container
*/
class Vendor {
  S32 a_id;
  std::string a_name;

public:

  struct Code {
    enum _v {
      None = -1, // Initialized
      Ietf = 0,
      Nokia = 94,
      Ericsson = 193,
      TresGPP = 10415,
      Telefonicaid = 5189,
      Etsi = 13019
    };

    anna_declare_enum(Code);

    /**
    * Vendor description
    * @param v Vendor code
    * @return Vendor name
    */
    static const char* asText(const Code::_v v) throw(anna::RuntimeException) {
      return asCString(v);
    }
  };


  Vendor() {};
  ~Vendor() {};


  // get
  S32 getId(void) const throw() { return a_id; }
  const S8 * getName(void) const throw() { return a_name.c_str(); }

  // helpers
  bool isVendorSpecific(void) const throw() { return (a_id > 0); }


  std::string asString(void) const throw();
  anna::xml::Node* asXML(anna::xml::Node* parent) const throw();

  // operators
  friend bool operator == (const Vendor & v1, const Vendor & v2) { return ((v1.getId() == v2.getId())); }

  // set
  void setId(const S32 & id) throw(anna::RuntimeException) {
    if(id < 0) throw anna::RuntimeException("Negative vendor-id not allowed", ANNA_FILE_LOCATION);

    a_id = id;
  }

  void setName(const std::string & n) throw(anna::RuntimeException) {
    if(n == "") throw anna::RuntimeException("Empty vendor-name string not allowed", ANNA_FILE_LOCATION);

    a_name = n;
  }
};


}
}
}


#endif
