// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_http_Response_hpp
#define anna_http_Response_hpp

#include <anna/http/Message.hpp>

namespace anna {

namespace http {

/**
   Clase que modela las respuestas HTTP segun la RFC 2616.
*/
class Response : public Message {
public:
  /**
     Constructor.
  */
  explicit Response() :
    Message(Message::Type::Response) { a_statusCode = 0; setStatusCode(200); }

  /**
     Devuelve el codigo de estado contenido en la respuesta.
     \return El codigo de estado contenido en la respuesta
  */
  int getStatusCode() const throw() { return a_statusCode; }

  /**
     Devuelve el campo ReasePhrase contenido en la respuesta.
     \return el campo ReasePhrase contenido en la respuesta.
  */
  const std::string& getReasonPhrase() const throw() { return a_reasonPhrase; }

  /**
     Establece el codigo de retorno de la respuesta HTTP. Si es un codigo reconocido por la RFC 2616
     tambien establece el ReasonPhrase por defecto.

     Los codigos de respuesta reconocidos por la RFC 2616 son:
     \code
        100: Continue
        101: Switching Protocols
        200: OK
        201: Created
        202: Accepted
        203: Non-Authoritative Information
        204: No Content
        205: Reset Content
        206: Partial Content
        300: Multiple Choise
        301: Moved Permanently
        302: Found
        303: See Other
        304: Not Modified
        305: Use proxy
        307: Temporary Redirect
        400: Bad Request
        401: Unautorized
        402: Payment Required
        403: Forbidden
        404: Not found
        405: Method Not Allowed
        406: Not Acceptable
        407: Proxy Authentication Required
        408: Request Time-out
        409: Conflict
        410: Gone
        411: Length Required
        412: Precondition Failed
        413: Request Entity Too Large
        414: Request-URI Too Large
        415: Unsupported Media Type
        416: Requested range not satisfiable
        417: Expectation Failed
        500: Internal Server Error
        501: Not Implemented
        502: Bad Gateway
        503: Service Unavailable
        504: Gateway Time-out
        505: HTTP Version not supported
     \endcode
  */
  void setStatusCode(const int statusCode) throw();

  /**
     Establece la frase asociada a la respuesta.
     \param reasonPhrase Frase asociada a la respuesta.
  */
  void setReasonPhrase(const std::string& reasonPhrase) throw() { a_reasonPhrase = reasonPhrase; }

  /**
     Establece la frase asociada a la respuesta.
     \param reasonPhrase Frase asociada a la respuesta.
  */
  void setReasonPhrase(const char* reasonPhrase) throw() { a_reasonPhrase = reasonPhrase; }

  /**
     Devuelve una cadena con toda la informacion relevante de este objeto.
     \return una cadena con toda la informacion relevante de este objeto.
  */
  std::string asString() const throw();

private:
  int a_statusCode;
  std::string a_reasonPhrase;

  std::string codeStartLine() const throw(anna::RuntimeException);
};

}
}

#endif

