// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <unistd.h>
#include <netdb.h>

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 64
#pragma Definiendo MAXHOSTNAMELEN
#endif

#include <anna/comm/functions.hpp>
#include <anna/comm/Application.hpp>
#include <anna/comm/Communicator.hpp>
#include <anna/comm/Service.hpp>
#include <anna/comm/Server.hpp>
#include <anna/core/tracing/Logger.hpp>

using namespace std;
using namespace anna;

comm::Application& comm::functions::getApp()
throw(RuntimeException) {
  if(comm::Application::st_application == NULL)
    throw RuntimeException("No Application class has been defined", ANNA_FILE_LOCATION);

  if(comm::Application::st_application->supportCommunication() == false)
    throw RuntimeException("Defined Application class has no communications support. Must inherit from anna::comm::Application or superior", ANNA_FILE_LOCATION);

  return *(static_cast <comm::Application*>(comm::Application::st_application));
}

string comm::functions::getHostName()
throw(RuntimeException) {
  char hostName [MAXHOSTNAMELEN];

  if(gethostname(hostName, MAXHOSTNAMELEN) != 0)
    throw RuntimeException("Cannot obtain the hostname", errno, ANNA_FILE_LOCATION);

  return string(hostName);
}




std::string comm::functions::resolveIP(const char* hostname)
throw(RuntimeException) {
  std::string result;
  struct hostent *host;

  if((host = gethostbyname(hostname)) == NULL) {
    string msg("comm::functions::resolveIP | Host to resolve: ");
    msg += hostname;
    msg += " | ";
    msg += hstrerror(h_errno);
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(host->h_addrtype != AF_INET) {
    string msg("comm::functions::resolveIP | Host to resolve: ");
    msg += hostname;
    msg += " | Address type unsupported";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  LOGDEBUG(
    string msg("comm::functions::resolveIP | Host to resolve: ");
    msg += hostname;

  if(host->h_name != NULL) { msg += " | Official name: "; msg += host->h_name; }
if(host->h_aliases[0] != NULL) {
  msg += " | Aliases:";

  for(int i = 0; host->h_aliases[i] != NULL; i ++) { msg += " "; msg += host->h_aliases[i]; }
  }
  msg += " | Address type: ";
  msg += functions::asString(host->h_addrtype);
  msg += " | Address length: ";
  msg += functions::asString(host->h_length);

  if(host->h_addr_list[0] != NULL) {
  msg += " | IPs:";

  for(int i = 0; host->h_addr_list[i] != NULL; i ++) { msg += " "; msg += inet_ntoa(*reinterpret_cast <in_addr*>(host->h_addr_list[i])); }
  }
  Logger::debug(msg, ANNA_FILE_LOCATION);
  );

  if(host->h_addr_list[0] == NULL) {
    string msg("comm::functions::resolveIP | Host to resolve: ");
    msg += hostname;
    msg += " | Address not resolved by the system";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  // Assignment:
  result = inet_ntoa(*reinterpret_cast <in_addr*>(host->h_addr_list[0]));   // inet_ntoa (const in_addr_t& address)
  return result;
}

