// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/mt/Guard.hpp>
#include <anna/core/tracing/TraceMethod.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/comm/internal/LocalConnection.hpp>
#include <anna/comm/Communicator.hpp>
#include <anna/comm/Device.hpp>
#include <anna/comm/ServerSocket.hpp>
#include <anna/comm/ClientSocket.hpp>
#include <anna/comm/Receiver.hpp>
#include <anna/comm/CongestionController.hpp>

#include <anna/comm/handler/LocalConnection.hpp>

using namespace std;
using namespace anna;

void comm::handler::LocalConnection::initialize()
throw(RuntimeException) {
  LOGMETHOD(TraceMethod traceMethod("comm::handler::LocationConnection", "initialize", ANNA_FILE_LOCATION));

  if(a_localConnection == NULL) {
    string msg(asString());
    msg += " | comm::LocalConnection was not established";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(a_localConnection->getClientSocket()  == NULL) {
    string msg(asString());
    msg += " | comm::ClientSocket was not established";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  setfd(a_localConnection->getClientSocket()->getfd());
  CongestionController::instantiate().incrementIncomingSocket();
}

comm::ClientSocket* comm::handler::LocalConnection::getClientSocket()
throw() {
  return (a_localConnection != NULL) ? a_localConnection->getClientSocket() : NULL;
}

/*
 * Se invoca desde Communicator::detach.

<Method Id="comm::handler::MetaClientSocket::apply" File="handler.db/comm.handler.MetaClientSocket.cc" Line="45" LineNo="281">
    <Guard Id="0x68ccd0" Name="comm::ClientSocket from comm::handler::MetaClientSocket::apply" LineNo="283"/>
         <Guard Id="0x68c6a0" Name="comm::Communicator::detach" LineNo="289">
         <Method Id="comm::handler::LocationConnection::finalize" File="handler.db/comm.handler.LocalConnection.cc" Line="78" LineNo="293">
            <Guard Id="0x68ccd0" Name="comm::Socket::close" LineNo="295">
                  <Guard Id="0x6818e0" Name="PN7anna4comm16TransportFactoryE" LineNo="297"/>
             </Guard>
        <Guard Id="0x68c678" Name="N7anna12SafeRecyclerINS_4comm15LocalConnectionENS_9AllocatorIS2_EEEE::release (T*)" LineNo="305"/>
        </Method>
     </Guard>
</Method>

 */
void comm::handler::LocalConnection::finalize()
throw() {
  LOGMETHOD(TraceMethod traceMethod("comm::handler::LocationConnection", "finalize", ANNA_FILE_LOCATION));

  if(a_localConnection == NULL)
    return;

  comm::ClientSocket* clientSocket = a_localConnection->getClientSocket();

  if(clientSocket != NULL) {
    // Notificacion al Receiver (EDU Julio 2012)
    Receiver* receiver;

    if((receiver = clientSocket->getReceiver()) != NULL)
      receiver->eventBreakLocalConnection(*clientSocket);

    a_communicator->eventBreakLocalConnection(*clientSocket);
    clientSocket->close();
  }

  comm::ServerSocket* serverSocket = a_localConnection->getServerSocket();

  if(serverSocket != NULL) {
    // serverSocket->eventBreakLocalConnection (a_localConnection); // @Eduardo (independence from communicator)
    //
    // Este lo quito porque ya tenemos el Receiver del clientSocket y eso cubre los dos tipos de politica de gestion de conexiones:
    //
    // (1) Un proceso servidor que admite N conexiones puede controlar las gestionadas mediante cierre/apertura del servidor de escucha (ServerSocket)
    //     Cuando llega al mximo configurado, hace un detach del comunicador (desaparece el LISTEN): communicator::detach(anna::comm::ServerSocket*)
    //     Cuando se rompe una conexion, se hace el attach del comunicador (aparece de nuevo el LISTEN): communicator::attach(anna::comm::ServerSocket*)
    //
    // (2) Un proceso servidor que admite N conexiones puede controlar las gestionadas mediante control de aceptacion devolviendo true o false al metodo
    //     virtual (eventAcceptConnection), void comm::handler::ServerSocket::apply ():
    //
    //               bool accept = a_communicator->eventAcceptConnection (*localConnection->getClientSocket ());
    //
    //               // @Eduardo (independence from communicator)
    //               if (accept) accept = a_serverSocket->eventAcceptConnection(*localConnection->getClientSocket ());
    //
    // Problemas:
    // (1) En este caso, si hemos ocupado todas las conexiones y no tenemos escucha, el ServerSocket ha desaparecido (es NULL) entonces NO INVOCARIA AL
    //     'serverSocket->eventBreakLocalConnection (a_localConnection);' porque no pasa por este bloque (serverSocket es NULL) y la aplicacin no puede ser notificada.
    //     Pero si la aplicacion establece un receiver al clientSocket recibido en el eventAcceptConnection, si sera notificada. Quito por lo tanto la notificacion
    //     'serverSocket->eventBreakLocalConnection (a_localConnection)', porque si la politica es la (2), la aplicacin sera notificada dos veces, lo cual no es deseable.
    //
    // (2) El problema de este metodo es el consumo de recursos, y sobre todo, que el cliente va a conectar y enviar paquetes que luego se sabe que no
    //     van a ninguna parte porque el accept devolver un false y se cerrara dicha conexion. Es preferible que el cliente vea un connection refused
    //     por no existir un LISTEN
    //
    serverSocket->release(a_localConnection);
  }

  a_localConnection = NULL;
  CongestionController::instantiate().decrementIncomingSocket();
}

string comm::handler::LocalConnection::asString() const
throw() {
  string result("comm::handler::LocalConnection { ");
  result += comm::Handler::asString();
  result += " | ";
  result += functions::asString(a_localConnection);
  return result += " }";
}

xml::Node* comm::handler::LocalConnection::asXML(xml::Node* parent) const
throw() {
  xml::Node* result = parent->createChild("comm.handler.LocalConnection");
  comm::Handler::asAttribute(result);

  if(a_localConnection)
    a_localConnection->asXML(result);

  return result;
}


