// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/core/oam/Module.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/oam/Configuration.hpp>
#include <anna/core/oam/CounterScope.hpp>
#include <anna/core/oam/CounterRecorder.hpp>

#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceWriter.hpp>
#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/functions.hpp>
#include <anna/xml/xml.hpp>

// Standard
#include <cstdarg>


#define UNDEFINED_EVENT_DESCRIPTION "<undefined>"


//******************************************************************************
//----------------------------------------------------------------------- Module
//******************************************************************************

//------------------------------------------------------------------------------
//-------------------------------------------------------------- Module::~Module
//------------------------------------------------------------------------------
anna::oam::Module::~Module() {
  for(scope_iterator scope_it = scope_begin(); scope_it != scope_end(); scope_it++)
    delete (scope(scope_it));
}


//------------------------------------------------------------------------------
//----------------------------------------------------------- Module::getScope()
//------------------------------------------------------------------------------
anna::oam::CounterScope *anna::oam::Module::getScope(const int &id) throw() {
  scope_iterator it = scope_find(id);
  return ((it != scope_end()) ? scope(it) : NULL);
}

////------------------------------------------------------------------------------
////------------------------------------------------------- Module::alarm_remove()
////------------------------------------------------------------------------------
//bool anna::oam::Module::alarm_remove(const int &key) throw() {
//   alarm_iterator it = alarm_find(key);
//   if (it != alarm_end()) {
//      a_alarms.erase(key);
//      return true;
//   }
//
//   return false;
//}
//
////------------------------------------------------------------------------------
////----------------------------------------------------- Module::counter_remove()
////------------------------------------------------------------------------------
//bool anna::oam::Module::counter_remove(const int &key) throw() {
//   counter_iterator it = counter_find(key);
//   if (it != counter_end()) {
//      a_counters.erase(key);
//      return true;
//   }
//
//   return false;
//}


// public:


//------------------------------------------------------------------------------
//----------------------------------------------------- Module::enableCounters()
//------------------------------------------------------------------------------
void anna::oam::Module::enableCounters(void) throw() {
  a_counters_enabled = true;
  LOGDEBUG(anna::Logger::debug("Scoped counters ENABLED", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//---------------------------------------------------- Module::disableCounters()
//------------------------------------------------------------------------------
void anna::oam::Module::disableCounters(void) throw() {
  a_counters_enabled = false;
  LOGDEBUG(anna::Logger::debug("Scoped counters DISABLED", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//------------------------------------------------------- Module::enableAlarms()
//------------------------------------------------------------------------------
void anna::oam::Module::enableAlarms(void) throw() {
  a_alarms_enabled = true;
  LOGDEBUG(anna::Logger::debug("Scoped alarms ENABLED", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Module::disableAlarms()
//------------------------------------------------------------------------------
void anna::oam::Module::disableAlarms(void) throw() {
  a_alarms_enabled = false;
  LOGDEBUG(anna::Logger::debug("Scoped alarms DISABLED", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//------------------------------------------------ Module::enableAlarmsPreffix()
//------------------------------------------------------------------------------
void anna::oam::Module::enableAlarmsPreffix(void) throw() {
  a_alarms_preffix_enabled = true;
  LOGDEBUG(anna::Logger::debug("Alarm preffix module components SHOWN", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//------------------------------------------------- Module::enableAlarmsSuffix()
//------------------------------------------------------------------------------
void anna::oam::Module::enableAlarmsSuffix(void) throw() {
  a_alarms_suffix_enabled = true;
  LOGDEBUG(anna::Logger::debug("Alarm suffix module components SHOWN", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//----------------------------------------------- Module::disableAlarmsPreffix()
//------------------------------------------------------------------------------
void anna::oam::Module::disableAlarmsPreffix(void) throw() {
  a_alarms_preffix_enabled = false;
  LOGDEBUG(anna::Logger::debug("Alarm preffix module components HIDDEN", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//------------------------------------------------ Module::disableAlarmsSuffix()
//------------------------------------------------------------------------------
void anna::oam::Module::disableAlarmsSuffix(void) throw() {
  a_alarms_suffix_enabled = false;
  LOGDEBUG(anna::Logger::debug("Alarm suffix module components HIDDEN", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//--------------------------------- Module::getDefaultInternalAlarmDescription()
//------------------------------------------------------------------------------
std::string anna::oam::Module::getDefaultInternalAlarmDescription(const int & type) const throw() {
  return UNDEFINED_EVENT_DESCRIPTION;
}


//------------------------------------------------------------------------------
//------------------------------- Module::getDefaultInternalCounterDescription()
//------------------------------------------------------------------------------
std::string anna::oam::Module::getDefaultInternalCounterDescription(const int & type) const throw() {
  return UNDEFINED_EVENT_DESCRIPTION;
}


//------------------------------------------------------------------------------
//---------------------------------------------- Module::alarmComponentsToText()
//------------------------------------------------------------------------------
std::string anna::oam::Module::alarmComponentsToText(const std::vector<std::string> & components, const std::string & psL, const std::string & psS, const std::string & psR) const throw() {
  if(components.size() == 0) return ("");

  std::vector<std::string>::const_iterator it_min(components.begin());
  std::vector<std::string>::const_iterator it_max(components.end());
  std::vector<std::string>::const_iterator it, it_last = it_max - 1;
  std::string result = psL;

  for(it = it_min; it != it_last; it++) { result += (*it); result += psS; }

  result += (*it_last);
  result += psR;
  return (result);
}


//------------------------------------------------------------------------------
//------------------------------ Module::getAlarmPreffixSuffixAndZoneSeparator()
//------------------------------------------------------------------------------
void anna::oam::Module::getAlarmPreffixSuffixAndZoneSeparator(std::string & preffix, std::string & suffix, char & zS) const throw() {
  Configuration &conf = Configuration::instantiate();
  const std::vector<std::string> * globalPreffixComponents = conf.getAlarmPreffixComponents();
  const std::vector<std::string> * globalSuffixComponents = conf.getAlarmSuffixComponents();
  std::vector<std::string> preffixComponents, suffixComponents, compsAux;

  if(globalPreffixComponents) preffixComponents = *globalPreffixComponents;

  if(globalSuffixComponents) suffixComponents = *globalSuffixComponents;

  if(a_alarms_preffix_enabled) {
    // Read from virtual and append to global configuration ones:
    readAlarmPreffixComponents(compsAux);

    if(compsAux.size() != 0) preffixComponents.insert(preffixComponents.begin(), compsAux.begin(), compsAux.end());
  }

  if(a_alarms_suffix_enabled) {
    // Read from virtual and append to global configuration ones:
    compsAux.clear();
    readAlarmSuffixComponents(compsAux);

    if(compsAux.size() != 0) suffixComponents.insert(suffixComponents.begin(), compsAux.begin(), compsAux.end());
  }

  // Build final preffix and suffix from its components:
  std::string psL, psS, psR;
  conf.getAlarmTextDelimiters(zS, psL, psS, psR);
  preffix = alarmComponentsToText(preffixComponents, psL, psS, psR);
  suffix = alarmComponentsToText(suffixComponents, psL, psS, psR);
}




//------------------------------------------------------------------------------
//--------------------------------------------- Module::initializeCounterScope()
//------------------------------------------------------------------------------
void anna::oam::Module::initializeCounterScope(const int & scopeId, const std::string & description) throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod tttm("anna::oam::Module", "initializeCounterScope", ANNA_FILE_LOCATION));

  // Order of use:
  if(a_counters.size() != 0)  // any counter have been registered
    throw anna::RuntimeException("After use of counter registration can't initialize more scope ids!. Do initializations firstly", ANNA_FILE_LOCATION);

  // Scope range: 0 - 99
  if(scopeId < 0 || scopeId >= MaxScope) {
    std::string msg = "Scope Id must be in range [0 - ";
    msg += anna::functions::asString(MaxScope - 1); // 99
    msg += "]";
    throw anna::RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  // Check module scopes:
  if(getScope(scopeId)) {
    std::string msg = "Scope Id '";
    msg += anna::functions::asString(scopeId);
    msg += "' has already been registered in this module!";
    throw anna::RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  // Create scope:
  bool missingScopeDescription = (description == "");
  LOGWARNING(

  if(missingScopeDescription && a_scopes.size() != 0) {
  anna::Logger::warning("This is not the first initialized scope. Perhaps you should provide specific description better than general module name ...", ANNA_FILE_LOCATION);
  }
  );
  const char * c_description = (missingScopeDescription ? getName() : description.c_str());
  a_active_counter_scope = new CounterScope(scopeId, c_description);
  a_scopes[scopeId] = a_active_counter_scope;
}


//------------------------------------------------------------------------------
//---------------------------------------------- Module::setActiveCounterScope()
//------------------------------------------------------------------------------
void anna::oam::Module::setActiveCounterScope(const int & scopeId) throw() {
  anna::oam::CounterScope *scope = getScope(scopeId);

  if(!scope) {
    anna::Logger::error(anna::functions::asString("Provided scope id '%d' can't be activated, because you must initialize it first", scopeId), ANNA_FILE_LOCATION);
    return;
  }

  a_active_counter_scope = scope;
}


//------------------------------------------------------------------------------
//---------------------------------------------------- Module::registerCounter()
//------------------------------------------------------------------------------
void anna::oam::Module::registerCounter(const int & type, const std::string & description, const int & offset) throw(anna::RuntimeException) {
  // Handler-specific
  a_handler->registerCounter(this, type, description, offset);

  // Order of use:
  if(a_scopes.size() == 0)  // at least one scope must be initialized
    throw anna::RuntimeException("Before counter registration, at least one counter scope must be initialized at oam module", ANNA_FILE_LOCATION);

  // Check type existence:
  const_counter_iterator counter_it = counter_find(type);

  if(counter_it != counter_end()) {
    std::string msg = "There is a former counter registered with enum value (counter type) = ";
    msg += anna::functions::asString(type);
    throw anna::RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  // If description is empty, get the default defined
  std::string _description = ((description != "") ? description : getDefaultInternalCounterDescription(type));
  counter_data_t aux;
  aux.Offset = offset;
  aux.Description = _description;
  // Register for all initialized scopes:

  for(scope_iterator scope_it = scope_begin(); scope_it != scope_end(); scope_it++) {
    int scopeId = (*scope_it).first;
    anna::oam::CounterScope * counterScope = scope(scope_it);
    counterScope->create(offset, _description.c_str());
    LOGDEBUG
    (
      std::string msg = "Added counter '";
      msg += _description;
      msg += "' over scope '";
      msg += counterScope->getName();
      msg += "': ScopeId/Offset [RealCounterId] = ";
      msg += anna::functions::asString(scopeId); msg += "/";
      msg += anna::functions::asString(offset); msg += " [";
      msg += anna::functions::asString((scopeId * anna::oam::CounterScope::MaxCounter) + offset); msg += "]";
      anna::Logger::debug(msg, ANNA_FILE_LOCATION);
    );
  }

  a_counters[type] = aux;
}



//------------------------------------------------------------------------------
//------------------------------------------------------ Module::registerAlarm()
//------------------------------------------------------------------------------
void anna::oam::Module::registerAlarm(const int & type, const std::string &description, const int & externalId, const std::string & dynamicVariablesCSL, const int & activationId, const int & cancellationId) throw(anna::RuntimeException) {
  // Handler-specific
  a_handler->registerAlarm(this, type, description, externalId, dynamicVariablesCSL, activationId, cancellationId);
  // Check type existence:
  const_alarm_iterator alarm_it = alarm_find(type);

  if(alarm_it != alarm_end()) {
    std::string msg = "There is a former alarm registered with enum value (alarm type) = ";
    msg += anna::functions::asString(type);
    throw anna::RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  // If description is empty, get the default defined
  std::string _description = ((description != "") ? description : getDefaultInternalAlarmDescription(type));
  alarm_data_t aux;
  aux.ExternalId = externalId;
  aux.DynamicVariablesCSL = dynamicVariablesCSL;
  aux.ActivationId = activationId;
  aux.CancellationId = cancellationId;
  aux.Description = _description;
  a_alarms[type] = aux;
  LOGDEBUG
  (
    std::string msg = "Added alarm '";
    msg += _description;
    msg += "': externalId[dynamicVariablesCSL]/activationId";

    if(cancellationId != -1) msg += "/cancellationId";
    msg += " = ";
    msg += anna::functions::asString(externalId); msg += "[";
    msg += anna::functions::asString(dynamicVariablesCSL); msg += "]/";
    msg += anna::functions::asString(activationId);
  if(cancellationId != -1) {
    msg += "/";
    msg += anna::functions::asString(cancellationId);
    }
  anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Module::activateAlarm()
//------------------------------------------------------------------------------
void anna::oam::Module::alarmEvent(bool activation, const int & type, va_list argList) const throw() {
  // Preffix/Suffix and separator:
  std::string userPreffix, userSuffix; char separator;
  getAlarmPreffixSuffixAndZoneSeparator(userPreffix, userSuffix, separator);
  const char *preffix = ((userPreffix != "") ? userPreffix.c_str() : NULL);
  const char *suffix = ((userSuffix != "") ? userSuffix.c_str() : NULL);
  va_list ap;
  va_copy(ap, argList); // http://julipedia.meroh.net/2011/09/using-vacopy-to-safely-pass-ap.html
  a_handler->alarmEvent(this, preffix, suffix, separator, activation, type, ap);
  va_end(ap);
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Module::activateAlarm()
//------------------------------------------------------------------------------
void anna::oam::Module::activateAlarm(int type, ...) const throw(anna::RuntimeException) {
//   LOGMETHOD(anna::TraceMethod tttm("anna::oam::Module", "activateAlarm", ANNA_FILE_LOCATION));

  // Checkings
  if(!a_alarms_enabled) {
    LOGDEBUG
    (
      std::string msg = "Alarm activation ignored over module '";
      msg += getName();
      msg += "': alarms are disabled";
      anna::Logger::debug(msg, ANNA_FILE_LOCATION);
    );
    return;
  }

  // User alarm event
  va_list argList;
  va_start(argList, type);
  alarmEvent(true, type, argList);
  va_end(argList);
}


//------------------------------------------------------------------------------
//-------------------------------------------------------- Module::cancelAlarm()
//------------------------------------------------------------------------------
void anna::oam::Module::cancelAlarm(int type, ...) const throw(anna::RuntimeException) {
//   LOGMETHOD(anna::TraceMethod tttm("anna::oam::Module", "cancelAlarm", ANNA_FILE_LOCATION));

  // Checkings
  if(!a_alarms_enabled) {
    LOGDEBUG
    (
      std::string msg = "Alarm cancellation ignored over module '";
      msg += getName();
      msg += "': alarms are disabled";
      anna::Logger::debug(msg, ANNA_FILE_LOCATION);
    );
    return;
  }

  // User alarm event
  va_list argList;
  va_start(argList, type);
  alarmEvent(false, type, argList);
  va_end(argList);
}


//------------------------------------------------------------------------------
//-------------------------------------------------------------- Module::count()
//------------------------------------------------------------------------------
void anna::oam::Module::count(const int & type, const int & amount) throw(anna::RuntimeException) {
//   LOGMETHOD(anna::TraceMethod tttm("anna::oam::Module", "count", ANNA_FILE_LOCATION));

  // Checkings
  if(!a_counters_enabled) {
    LOGDEBUG
    (
      std::string msg = "Count operation ignored over module '";
      msg += getName();
      msg += "': counters are disabled";
      anna::Logger::debug(msg, ANNA_FILE_LOCATION);
    );
    return;
  }

  a_handler->counterEvent(this, type, amount);
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Module::resetCounters()
//------------------------------------------------------------------------------
int anna::oam::Module::resetCounters(const int & scopeId) throw() {
  LOGMETHOD(anna::TraceMethod tttm("anna::oam::Module", "resetCounters", ANNA_FILE_LOCATION));
  int result = 0; // affected number

  for(scope_iterator scope_it = scope_begin(); scope_it != scope_end(); scope_it++)
    result += scope(scope_it)->resetAccValues();

  return result;
}


//------------------------------------------------------------------------------
//------------------------------------- Module::RecordingGuard::RecordingGuard()
//------------------------------------------------------------------------------
anna::oam::Module::RecordingGuard::RecordingGuard(anna::oam::Module* module) :
  a_module(module) {                                                
  module->a_counterRecording = true;                                               
}                                                                                   
                                                                                    

//------------------------------------------------------------------------------
//------------------------------------ Module::RecordingGuard::~RecordingGuard()
//------------------------------------------------------------------------------
anna::oam::Module::RecordingGuard::~RecordingGuard() {                            
  a_module->a_counterRecording = false;                                            
}                                                                                   


//------------------------------------------------------------------------------
//----------------------------------------------------- Module::recordCounters()
//------------------------------------------------------------------------------
void anna::oam::Module::recordCounters() throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod tttm("anna::oam::Module", "recordCounters", ANNA_FILE_LOCATION));

  if(a_counterRecorder == NULL)
    throw RuntimeException("anna::oam::Module has no counter recorder associated", ANNA_FILE_LOCATION);

  LOGDEBUG(
    std::string msg("Recording counters | ");
    msg += a_counterRecorder->asString();
    anna::Logger::write(Logger::Debug, msg, ANNA_FILE_LOCATION)
  );
  RecordingGuard guard(this);

  a_counterRecorder->open();
  CounterScope* cscope = NULL;

  try {
    for(scope_iterator scope_it = scope_begin(); scope_it != scope_end(); scope_it++) {
      cscope = scope(scope_it);
      if(cscope == NULL) continue;

      Guard csGuard(cscope, "oam::CounterScope from oam::Module::record");

      for(int icounter = 0; icounter < CounterScope::MaxCounter; icounter ++) {
        Counter* counter = cscope->a_counters [icounter];

        if(counter == NULL)
          continue;

        if(counter->getValue() == 0)
          continue;

        a_counterRecorder->apply(*counter);
        counter->reset();
      }
    }

    a_counterRecorder->close();
  } catch(anna::RuntimeException&) {
    a_counterRecorder->close();
    throw;
  }
}

//------------------------------------------------------------------------------
//----------------------------------------------------------- Module::asString()
//------------------------------------------------------------------------------
std::string anna::oam::Module::asString(void) const throw() {
  std::string trace;
  trace = "Module name: '";
  trace += getName();
  trace += "'";
  trace += "\n\nCOUNTERS"; trace += "\n--------";
  trace += "\nScoped counters "; trace += a_counters_enabled ? "Enabled" : "Disabled";
  trace += "\nCounter Scopes: "; trace += anna::functions::asString(a_scopes.size());

  for(const_scope_iterator scope_it = scope_begin(); scope_it != scope_end(); scope_it++) {
    trace += "\n\nCounter Scope:";
    int scopeId = (*scope_it).first;
    trace += " Id: "; trace += anna::functions::asString(scopeId);
    trace += " | Description: '"; trace += scope(scope_it)->getName(); trace += "'";
    trace += "\nRegistered counters:\n";
    counter_data_t * ptrCounterData;

    for(const_counter_iterator cnt_it = counter_begin(); cnt_it != counter_end(); cnt_it++) {
      ptrCounterData = (counter_data_t *) & ((*cnt_it).second);
      // Counter:
      trace += "\n\tType: "; trace += anna::functions::asString((*cnt_it).first);
      trace += " | Description: '"; trace += ptrCounterData->Description; trace += "'";
      int offset = ptrCounterData->Offset;
      int realId = (1000 * scopeId) + offset;
      trace += " | ScopeId/Offset: "; trace += anna::functions::asString(scopeId); trace += "/"; trace += anna::functions::asString(offset);
      trace += " | RealId: "; trace += anna::functions::asString(realId);
      unsigned long long int accValue = scope(scope_it)->getAccValue(offset);

      if(accValue != 0ULL) {
        trace += " | AccumulatedAmount: ";
        trace += anna::functions::asString("%llu", accValue);
      }
    }
  }

  trace += "\n\nALARMS"; trace += "\n------";
  trace += "\nScoped alarms "; trace += a_alarms_enabled ? "Enabled" : "Disabled";
  trace += "\nPreffix alarm components: "; trace += a_alarms_preffix_enabled ? "Shown" : "Hidden";
  trace += "\nSuffix alarm components: "; trace += a_alarms_suffix_enabled ? "Shown" : "Hidden";
  trace += "\nRegistered alarms:\n";

  for(const_alarm_iterator alrm_it = alarm_begin(); alrm_it != alarm_end(); alrm_it++) {
    int cancellationId = (*alrm_it).second.CancellationId;
    bool transferable = (cancellationId != -1);
    trace += "\n\tType: "; trace += anna::functions::asString((*alrm_it).first);
    trace += " | Description: '"; trace += (*alrm_it).second.Description;
    trace += transferable ? "' [transferable alarm]" : "'";
    trace += " | DatabaseId: "; trace += anna::functions::asString((*alrm_it).second.ExternalId);
    trace += " | dynamicVariablesCSL: "; trace += (*alrm_it).second.DynamicVariablesCSL;
    trace += " | ActivationId: "; trace += anna::functions::asString((*alrm_it).second.ActivationId);

    if(transferable) { trace += " | CancellationId: "; trace += anna::functions::asString(cancellationId); }
  }

  return (trace);
}



//------------------------------------------------------------------------------
//-------------------------------------------------------------- Module::asXML()
//------------------------------------------------------------------------------
anna::xml::Node* anna::oam::Module::asXML(anna::xml::Node* parent) const throw() {
  anna::xml::Node* result = parent->createChild("oam.Module");
  result->createAttribute("Name", getName());
  result->createAttribute("Counters", a_counters_enabled ? "Enabled" : "Disabled");
  anna::xml::Node* registeredCounterScopes = result->createChild("RegisteredCounterScopes");

  for(const_scope_iterator scope_it = scope_begin(); scope_it != scope_end(); scope_it++) {
    // Scope:
    anna::xml::Node* scopeNode = registeredCounterScopes->createChild("Scope");
    int scopeId = (*scope_it).first;
    scopeNode->createAttribute("Id", anna::functions::asString(scopeId));
    scopeNode->createAttribute("Description", scope(scope_it)->getName());
    // Counters:
    anna::xml::Node* registeredCounters = scopeNode->createChild("RegisteredCounters");
    counter_data_t * ptrCounterData;

    for(const_counter_iterator cnt_it = counter_begin(); cnt_it != counter_end(); cnt_it++) {
      ptrCounterData = (counter_data_t *) & ((*cnt_it).second);
      // Counter:
      anna::xml::Node* counter = registeredCounters->createChild("Counter");
      counter->createAttribute("Type", anna::functions::asString((*cnt_it).first));
      counter->createAttribute("Description", ptrCounterData->Description);
      int offset = ptrCounterData->Offset;
      int realId = (1000 * scopeId) + offset;
      counter->createAttribute("ScopeId", anna::functions::asString(scopeId));
      counter->createAttribute("Offset", anna::functions::asString(offset));
      counter->createAttribute("RealId", anna::functions::asString(realId));
      unsigned long long int accValue = scope(scope_it)->getAccValue(offset);

      if(accValue != 0ULL)
        counter->createAttribute("AccumulatedAmount", anna::functions::asString("%llu", accValue));
    }
  }

  result->createAttribute("Alarms", a_alarms_enabled ? "Enabled" : "Disabled");
  result->createAttribute("PreffixAlarmComponents", a_alarms_preffix_enabled ? "Shown" : "Hidden");
  result->createAttribute("SuffixAlarmComponents", a_alarms_suffix_enabled ? "Shown" : "Hidden");
  anna::xml::Node* registeredAlarms = result->createChild("RegisteredAlarms");

  for(const_alarm_iterator alrm_it = alarm_begin(); alrm_it != alarm_end(); alrm_it++) {
    anna::xml::Node* alarm;
    int cancellationId = (*alrm_it).second.CancellationId;
    bool transferable = (cancellationId != -1);
    alarm  = registeredAlarms->createChild("Alarm");
    alarm->createAttribute("Type", anna::functions::asString((*alrm_it).first));
    std::string desc = (*alrm_it).second.Description; desc += " ["; desc += transferable ? "transferable]" : "non transferable]";
    alarm->createAttribute("Description", desc);
    alarm->createAttribute("DatabaseId", anna::functions::asString((*alrm_it).second.ExternalId));
    alarm->createAttribute("DynamicVariablesCSL", (*alrm_it).second.DynamicVariablesCSL);
    alarm->createAttribute("ActivationId", anna::functions::asString((*alrm_it).second.ActivationId));

    if(transferable) alarm->createAttribute("CancellationId", anna::functions::asString(cancellationId));
  }

  return result;
}

